# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
__version__ = '$Revision: 1.23 $'[11:-2]

import os
from Acquisition import aq_base, aq_inner, aq_parent
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from zLOG import LOG, ERROR
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.Expression import Expression
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, _checkPermission, getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent, ManagePortal
from Products.CMFDefault.DiscussionTool import DiscussionTool

from Products.Archetypes.config import TOOL_NAME as ARCHETYPE_TOOL

from config import discussionSkin, discussion_configlet
from DiscussionFilter import ForumDiscussionFilter
from ForumContentTopic import ForumContentTopic

class DiscussionError( Exception ):
    pass

class DiscussionNotAllowed( Exception ):
    pass

class ForumDiscussionTool( UniqueObject, SimpleItem, ActionProviderBase ):
    """ """
    meta_type = 'ForumDiscussionTool'

    security = ClassSecurityInfo()

    global_allow = 0

    _actions = [ActionInformation(id='reply'
                                , title='Reply'
                                , action=Expression(
                text='string: ${object_url}/discussion_reply_form')
                                , condition=Expression(
                text='python: object is not None and ' +
                'portal.portal_discussion.isDiscussionAllowedFor(object)')
                                , permissions=('Reply to item',)
                                , category='object'
                                , visible=1
                                 )]

    _default = ''
    _default_vm = 'nested'
    
    only_published = 0
    
    _type_vm = {}
    _type_map  = {}

    def __init__(self):
        self.id = 'portal_discussion'

    def manage_afterAdd(self, item, container):
        skinstool = getToolByName(self, 'portal_skins')
        portal = getToolByName(self, 'portal_url').getPortalObject()

        for skinName in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skinName)
            path = [i.strip() for i in path.split(',')]
            if discussionSkin not in path:
                try:
                    path.insert(path.index('custom')+1, discussionSkin)
                except:
                    path.append(discussionSkin)

            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)

        try:
            cctool = getToolByName(self, 'portal_controlpanel')
            cctool.registerConfiglet(**discussion_configlet)
        except:
            pass

    def manage_beforeDelete(self, item, container):
        skinstool = getToolByName(self, 'portal_skins')
        portal = getToolByName(self, 'portal_url').getPortalObject()

        for skinName in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skinName)
            path = [i.strip() for i in path.split(',')]
            while discussionSkin in path:
                del path[path.index(discussionSkin)]

            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)

        try:
            cctool = getToolByName(self, 'portal_controlpanel')
            cctool.unregisterConfiglet(discussion_configlet['id'])
        except:
            pass
        
    security.declareProtected(ManagePortal, 'edit_tool')
    def edit_tool(self, default, default_vm, only_published):
        """ """
        request = self.REQUEST
        traverse = self.unrestrictedTraverse

        self.only_published = not not only_published
        self._default = default
        self._default_vm = default_vm

        map = {}
        map_vm = {}
        for tp in getToolByName(self, 'portal_types').listTypeInfo():
            id = tp.getId()
            f_var = 'tp_%s'%id
            vm_var = 'vm_%s'%id
            if request.has_key(f_var):
                forum = request[f_var]
                map[id] = forum
                
            if request.has_key(vm_var):
                vm = request[vm_var]
                map_vm[id] = vm

        self._type_map = map
        self._type_vm = map_vm

    security.declareProtected(View, 'DefaultForum')
    def DefaultForum(self):
        """ """
        return self._default

    security.declareProtected(View, 'DefaultViewMode')
    def DefaultViewMode(self):
        """ """
        return self._default_vm

    security.declareProtected(View, 'TypeForum')
    def TypeForum(self, tp):
        """ """
        try:
            return self._type_map.get(tp, '')
        except:
            return ''

    security.declareProtected(View, 'TypeViewMode')
    def TypeViewMode(self, tp, default=0):
        """ """
        try:
            vm = self._type_vm.get(tp, '')
            if vm or default:
                return vm
        except:
            pass

        return self._default_vm

    security.declarePublic('getViewMode')
    def getViewMode(self, content):
        """ """
        return self.TypeViewMode(content.getPortalTypeName())

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return available actions via tool.
        """
        return self._actions

    security.declarePublic( 'overrideDiscussionFor' )
    def overrideDiscussionFor(self, content, allowDiscussion):
        """
            Override discussability for a per object basis or clear and let
            the site default override.
        """
        mtool = getToolByName( self, 'portal_membership' )
        if not mtool.checkPermission( ModifyPortalContent, content ):
            raise "Unauthorized"

        if allowDiscussion is None or allowDiscussion == 'None':
            if hasattr(content, 'allow_discussion'):
                del content.allow_discussion
        else:
            content.allow_discussion = int(allowDiscussion)

    security.declarePublic( 'isDiscussionAllowedFor' )
    def isDiscussionAllowedFor( self, content ):
        '''
            Returns a boolean indicating whether a discussion is
            allowed for the specified content.
        '''

        allowed = 1
        object = content
        portal = aq_parent(aq_inner(self))

        while 1:
            object = aq_parent(aq_inner(object))

            filter = getattr(object, 'discussion_filters', None)

            if filter is not None:
                if not isinstance(filter, ForumDiscussionFilter):
                    object = filter
                    continue

                topic_rec = filter.isDiscussionAllowedFor(content, portal)
                if topic_rec['status'] == None:
                    # search next filter
                    object = filter
                else:
                    # return state topic
                    allowed = topic_rec['allowed']
                    break
            else:
                break

        if not allowed:
            return 0

        if self.only_published:
            try:
                if self.portal_workflow.getInfoFor(content,'review_state') != 'published':
                    return 0
            except:
                return 0
        
        if hasattr( content, 'allow_discussion' ):
            return content.allow_discussion
        typeInfo = getToolByName(self, 'portal_types').getTypeInfo( content )
        if typeInfo:
            return typeInfo.allowDiscussion()
        return 0

    security.declarePublic( 'getDiscussionFor' )
    def getDiscussionFor(self, content):
        """
            Return the talkback for content, creating it if need be.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        talkback = getattr( content, 'talkback', None )
        
        
        if not talkback:
            return self._createDiscussionFor( content )
        else:
            return self._getTopic(content, talkback)

    security.declarePublic( 'hasDiscussionFor' )
    def hasDiscussionFor(self, content):
        """
            Return the talkback for content, creating it if need be.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        talkback = getattr( content, 'talkback', None )
        if talkback is not None:
            try:
                topic_ob = getToolByName(self, ARCHETYPE_TOOL).lookupObject(talkback)
                return topic_ob.replies > 0
            except:
                pass
        return 0

    security.declarePrivate( '_getTopic' )
    def _getTopic(self, content, topic):
        forum = self._type_map.get(content.getPortalTypeName(), '')
        if not forum:
            forum = self._default

        forum_ob = getToolByName(self, ARCHETYPE_TOOL).lookupObject(forum)

        if forum_ob is None:
            return None

        try:
            topic_ob = getToolByName(self, ARCHETYPE_TOOL).lookupObject(topic)
        except:
            topic_ob = None

        if topic_ob is None:
            return self._createDiscussionFor( content )
        else:
            return topic_ob

    security.declarePrivate('_get_filter')
    def _get_filter(self, object):
        object = aq_inner(object)

        filter = getattr(object, 'discussion_filters')
        return filter

    security.declarePrivate( '_createDiscussionFor' )
    def _createDiscussionFor( self, content ):
        """ """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        object = content
        portal = aq_parent(aq_inner(self))
        
        while 1:
            object = aq_inner(object)
            
            filter = getattr(object, 'discussion_filters', None)
            if filter is not None:
                topic_rec = filter.createDiscussionFor(content, portal)
                if topic_rec['status'] == None:
                    # search next filter
                    object = filter
                else:
                    # return created topic
                    return topic_rec['topic']
            else:
                break

        forum = self._type_map.get(content.getPortalTypeName(), '')
        if not forum:
            forum = self._default

        forum_ob = getToolByName(self, ARCHETYPE_TOOL).lookupObject(forum)

        if forum_ob is None:
            #raise DiscussionError
            return None

        # create topic
        topic_ob = forum_ob.manage_addTopic(
            content.Title() or 'discussion', '', language=content.Language(),
            portal_type=ForumContentTopic.meta_type,
            content_path='/'.join(content.getPhysicalPath()),
            text_format=getattr(content,'text_format','text/plain'))
        
        if hasattr(content, 'talkback'):
            try:
                content.talkback.manage_beforeDelete(content.talkback, content)
            except:
                pass

        content.talkback = topic_ob.UID()

        return topic_ob


InitializeClass( ForumDiscussionTool )
