__version__ = '$Revision: 1.34 $'[11:-2]

import traceback
from cStringIO import StringIO
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
from Products.CMFBoard import config
from Products.CMFBoard.config import *
from Products.CMFBoard.ForumPermissions import *
from Products.CMFBoard.ForumWorkflow import \
     topic_workflow_id, topic_workflow, forum_workflow_id, forum_workflow, \
     message_workflow_id, message_workflow

def install(self):
    out = StringIO()

    portal = getToolByName(self, 'portal_url').getPortalObject()

    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    install_global_properties(self, out)
    
    # install validation
    form_tool = getToolByName(self, 'portal_form')
    for form, val in validators.items():
        form_tool.setValidators(form, val)

    nav_tool = getToolByName(self, 'portal_navigation')
    nav_tool.removeTransitionFrom('ForumMessage')
    
    # install navigation
    for tp, a1 in forms.items():
        for form, a2 in a1.items():
            for st, trans in a2.items():
                nav_tool.addTransitionFor(tp, form, st, trans)

    # install mail spool tool
    if not hasattr(portal, 'portal_mailspool'):
        try:
            portal.manage_addProduct['CMFBoard'].manage_addTool('CMFBoard Mail Spool', None)
            print >> out, "Successfully installed MailSpool Tool."
        except:
            print >> out, "Failed to install MailSpool Tool."

    dtool = getattr(self, 'portal_discussion', None)
    if (dtool is not None) and (dtool.meta_type == 'ForumDiscussionTool'):
        skinstool = getToolByName(self, 'portal_skins')

        for skinName in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skinName)
            path = [i.strip() for i in path.split(',')]
            if discussionSkin not in path:
                try:
                    path.insert(path.index('custom')+1, discussionSkin)
                except:
                    path.append(discussionSkin)

            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)

    try:
        self.archetype_tool.manage_updateSchema()
        print >> out, "Successfully updated archetypes schemas."
    except Exception, e:
        print >> out, "Faild to update archetypes schemas:", e

    
    install_external_methods(self, portal.portal_skins.custom, out)
    
    install_workflow(self)
    print >> out, "Successfully installed custom workflows."

    install_member_properties(self)
    print >> out, "Successfully installed member properties."

    install_catalog_indexes(self)
    print >> out, "Successfully installed catalog indexes."

    upgrade(self, out)
    print >> out, "Successfully upgraded old forums."

    # install left slots
    try:
        portal = getToolByName(self, 'portal_url').getPortalObject()
        left_slots = getattr(portal, 'left_slots', [])
        for slot in slots:
            if slot not in left_slots:
                try:
                    left_slots = left_slots[:1] + (slot,) + left_slots[1:]
                except:
                    left_slots += (slot,)
        portal.left_slots = left_slots
        print >> out, "Successfully installed left slots."
    except Exception, e:
        print >> out, "Faild to install left slots.", e

    install_actions(self, out)

    print >> out, "Successfully installed %s." % PROJECTNAME
    
    return out.getvalue()

def uninstall(self):
    out = StringIO()

    # uninstall configs
    if config.plone_ver == 2:
        self.portal_controlpanel.unregisterConfiglet(cmfboard_configlet['id'])
        self.portal_controlpanel.unregisterConfiglet(forum_preferences['id'])
    else:
        try:
            actions = self.portal_properties.listActions()
            for idx in range(len(actions)):
                if actions[idx].id == cmfboard_configlet['id']:
                    self.portal_properties.deleteActions((idx,))
                    break
        except:
            traceback.print_exc()

    # uninstall left_slots
    portal = getToolByName(self, 'portal_url').getPortalObject()
    left_slots = getattr(portal, 'left_slots', [])
    for slot in slots:
        try:
            left_slots.remove(slot)
        except:
            pass
    portal.left_slots = left_slots
    print >> out, "Successfully uninstalled left slots."

    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()

def install_actions(self, out):
    if config.plone_ver != 1:
        self.portal_controlpanel.registerConfiglet(**cmfboard_configlet)
        self.portal_controlpanel.registerConfiglet(**forum_preferences)
    else:
        self.portal_properties.addAction(**cmfboard_configlet)


    skinstool = self.portal_skins
    maillistSkin = 'cmfboard/fcMaillist'

    unique_emails = self.portal_properties.cmfboard_properties.unique_emails

    if unique_emails:
        for skinName in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skinName)
            path = [i.strip() for i in path.split(',')]
            if maillistSkin not in path:
                try:
                    path.insert(path.index('custom')+1, maillistSkin)
                except:
                    path.append(maillistSkin)

            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)
    else:
        for skinName in skinstool.getSkinSelections():
            path = skinstool.getSkinPath(skinName)
            path = [i.strip() for i in path.split(',')]
            while maillistSkin in path:
                del path[path.index(maillistSkin)]

            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)


def install_workflow(self):
    wtool = self.portal_workflow

    for r in forumRoles:
        try:
            wtool._addRole(r)
        except:
            pass

    if topic_workflow_id in wtool.objectIds():
        wtool._delObject(topic_workflow_id)

    wtool.manage_addWorkflow(id=topic_workflow_id, workflow_type=topic_workflow)
    wtool.setChainForPortalTypes(('ForumPoll',), topic_workflow_id)
    wtool.setChainForPortalTypes(('ForumTopic',), topic_workflow_id)
    wtool.setChainForPortalTypes(('ForumContentTopic',), topic_workflow_id)

    if forum_workflow_id in wtool.objectIds():
        wtool._delObject(forum_workflow_id)

    wtool.manage_addWorkflow(id=forum_workflow_id, workflow_type=forum_workflow)
    wtool.setChainForPortalTypes(('ForumNB',), forum_workflow_id)
    wtool.setChainForPortalTypes(('ForumFolder',), 'folder_workflow')

    if message_workflow_id in wtool.objectIds():
        wtool._delObject(message_workflow_id)

    wtool.manage_addWorkflow(id=message_workflow_id, workflow_type=message_workflow)
    wtool.setChainForPortalTypes(('ForumMessage',), message_workflow_id)

def install_member_properties(self):
    mdtool = getToolByName(self, 'portal_memberdata')
    for prop, tp, val in config.memberData_properties:
        if not mdtool.hasProperty(prop):
            mdtool._setProperty(prop, val, tp)

def install_global_properties(self, out):
    # install global properties
    if not hasattr(self.portal_properties, 'cmfboard_properties'):
        self.portal_properties.addPropertySheet(
            'cmfboard_properties', 'CMFBoard properties')

    props = self.portal_properties.cmfboard_properties
    for prop, tp, val in config.global_properties:
        if not props.hasProperty(prop):
            props._setProperty(prop, val, tp)
    print >> out, "Successfully installed global forum properties."

def install_catalog_indexes(self):
    ctool = getToolByName(self, 'portal_catalog')

    indexes = ctool.indexes()
    for index, tp in config.indexes:
        if index not in indexes:
            ctool.addIndex(index, tp)


def upgrade(self, out):
    ctool = self.portal_catalog
    wtool = self.portal_workflow

    brains = ctool(Type='Forum Folder')
    for brain in brains:
        ob = brain.getObject()
        try:
            ob.upgradeFolder()
        except Exception, e:
            print >> out, e
        
    brains = ctool(Type='Forum')
    for brain in brains:
        ob = brain.getObject()
        try:
            ob.upgradeForum()
        except Exception, e:
            print >> out, e

    brains = ctool(Type='ForumNB')
    for brain in brains:
        ob = brain.getObject()
        try:
            for role in forumRoles:
                ob._addRole(role)
                
            ctool.reindexObject(ob, 'forum_email')
        except Exception, e:
            print >> out, e

    # updare Topic state
    brains = self.portal_catalog(review_state='normal')
    for brain in brains:
        try:
            ob = brain.getObject()
            wtool.doActionFor(ob, 'publish')
        except:
            pass

    brains = self.portal_catalog(Type=('Forum Topic', 'Forum Poll',
                                       'Forum Content Topic'))
    for brain in brains:
        ob = brain.getObject()
        try:
            owner = ob.getOwner()
            ob.manage_setLocalRoles(owner.getUserName(), [topicOwner,])
            for r in  ob.rolesOfPermission(ModifyTopic):
                if (r['name'] == 'Owner') and (r['selected']=='SELECTED'):
                    self.manage_permission(ModifyTopic, (topicOwner,), 0)
            self.manage_permission(RequestReview, (topicOwner,), 0)
        except Exception, e:
            print >> out, e

    brains = self.portal_catalog(Type='Forum Message')
    for brain in brains:
        ob = brain.getObject()
        try:
            owner = ob.getOwner()
            ob.manage_setLocalRoles(owner.getUserName(), [replyOwner,])
            for r in  ob.rolesOfPermission(ModifyReply):
                if (r['name'] == 'Owner') and (r['selected']=='SELECTED'):
                    self.manage_permission(ModifyReply, (replyOwner,), 0)
            self.manage_permission(RequestReview, (replyOwner,), 0)
        except Exception, e:
            print >> out, e



# install ExternalMethods
def install_external_methods(self, folder, out):
    if not hasattr(folder, 'change_email_for_member'):
        try:
            manage_addExternalMethod(folder,'change_email_for_member',
                                     '', 'CMFBoard.utils','change_email_for_member')
            print >> out, "Successfully installed ExternalMethods."
        except:
            print >> out, "Failed to install ExternalMethods."

    if not hasattr(folder, 'check_member_email'):
        try:
            manage_addExternalMethod(folder,'check_member_email',
                                     '', 'CMFBoard.utils','check_member_email')
            print >> out, "Successfully installed check_member_email ExternalMethod."
        except:
            print >> out, "Failed to install check_member_email ExternalMethod."


    
def dump(self):
    print '-------------- begin dump ---------------------'
    print '----- object: %s ----------------'%'/'.join(self.getPhysicalPath())
    print '----- classes: %s ---------------'%str(self.__class__.__bases__)

    items = self.__dict__.items()
    items.sort()
    for key, val in items:
        print key, repr(val)


    print repr(self.file), self.file.getSize()

    print '-------------- end dump -----------------------'
    try:
        print self.getRelationships()
        print self.getRefs()
        print self.getBRefs()
        print self.archetype_tool.refs.get(self.UID(), [])
        print self.archetype_tool.bref.get(self.UID(), [])
    except:
        pass
    print '----------'
