# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
__version__ = '$Revision: 1.36 $'[11:-2]

import random, traceback, time

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from Products.Archetypes.public import *
from Products.Archetypes.Referenceable import Referenceable
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, AccessContentsInformation
from Products.CMFCore.utils import getToolByName, _checkPermission
from BTrees.OOBTree import OOBTree

from config import plone_ver
from ForumCatalog import *
from ForumPermissions import *
from Subscribeable import Subscribeable
from UserPreferences import UserPreferences
from PermissionsManagement import PermissionsManagement

try:
    import ForumPoll
    has_poll = 1
except ImportError, e:
    has_poll = 0

role_map = {
    0:forumManager,1:forumReviewer,2:forumWriter,3:replyWriter,4:forumReader }

version = '1.2'

view_modes = DisplayList([('flat', 'Flat'), ('nested', 'Nested'),
                         ('thread','Thread'), ('full_thread','Full thread')])
quickreply_forms = DisplayList([
    ('', 'Not use'),
    ('default', 'Default'),
    ('here/forum_quickreply_form/macros/edit_form', 'Simple form'),
    ('here/forum_blog_widget/macros/blog_form', 'Full form'),
    ])


class ForumNB(BaseBTreeFolder, Subscribeable, UserPreferences, PermissionsManagement):
    """fcForum Folder(ish)"""
    meta_type = 'ForumNB'
    archetype_name = 'ForumNB'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []
    content_icon = 'fcForum_icon.gif'

    schema = BaseSchema + Schema((
        StringField('id', required=1, mode='rw',
                    accessor='getId', mutator='setId', default=None,
                    write_permission = ManageForum,
                    widget=IdWidget(label_msgid='label_name',
                                    description_msgid='help_name',
                                    i18n_domain='plone')),
        StringField('title', required=1, default='Forums',
                    searchable=1,
                    write_permission=ManageForum,
                    widget=StringWidget(label_msgid='label_title',
                                        description_msgid='help_title',
                                        i18n_domain='plone')),
        BooleanField('moderated', required=0, default=0,
                     write_permission=ManageForum,
                     widget=BooleanWidget()),
        BooleanField('show_wizards', required=0, default=1,
                     write_permission=ManageForum,
                     widget=BooleanWidget(label='Show wizards',
                                          label_msgid='label_show_wizards',
                                          description_msgid='help_show_wizards',
                                          i18n_domain='plone')),
        StringField('quickreply_form', required=0, default='default',
                    vocabulary=quickreply_forms,
                    write_permission=ManageForum,
                    widget=SelectionWidget(label='Quick reply form',
                                           label_msgid='label_quickreply_form',
                                           description_msgid='help_quickreply_form',
                                           i18n_domain='CMFBoard')),
        StringField('view_mode', required=0, default='flat',
                    vocabulary=view_modes,
                    write_permission=ManageForum,
                    widget=SelectionWidget(label='Default view mode',
                                           label_msgid='label_view_mode',
                                           description_msgid='help_view_mode',
                                           i18n_domain='CMFBoard')),
        IntegerField('file_size', required=1, default=100000,
                     write_permission=ManageForum,
                     widget=StringWidget(label='Max file size (bytes)',
                                         label_msgid='label_file_size',
                                         description_msgid='help_file_size',
                                         i18n_domain='CMFBoard')),
        IntegerField('image_size', required=1, default=150000,
                     write_permission=ManageForum,
                     widget=StringWidget(label='Max image size (bytes)',
                                         label_msgid='label_image_size',
                                         description_msgid='help_image_size',
                                         i18n_domain='CMFBoard')),
        TextField('description', required=0, default='',
                  searchable=1,
                  write_permission=ManageForum,
                  widget=TextAreaWidget(label_msgid='label_description',
                                        description_msgid='help_description',
                                        i18n_domain='plone')),
        StringField('forum_email', required=0, default='',
                    write_permission=ManageForum,
                    widget=StringWidget(label='Forum email',
                                        label_msgid='label_forum_email',
                                        description_msgid='help_forum_email',
                                        i18n_domain='CMFBoard')),
        ))

    if plone_ver == 1:
        actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'forumnb_view',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'portal_form/base_edit',
              'permissions': (ManageForum,)
              },
            { 'id': 'forum_permissions',
              'name': 'Permissions',
              'action': 'forum_permissions',
              'permissions': (ManageForum,),
              }
            )
    else:
        actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/forumnb_view',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/portal_form/base_edit',
              'permissions': (ManageForum,)
              },
            { 'id': 'forum_permissions',
              'name': 'Permissions',
              'action': 'string:$object_url/forum_permissions',
              'permissions': (ManageForum,),
              }
            )
    
    security = ClassSecurityInfo()

    moderated = 0
    show_wizards = 1
    forum_email = ''
    quickreply_form = 'default'

    __ac_roles__ = forumRoles
    
    security.declareProtected(ManageForum, 'processForm')

    security.declarePublic('isModerated')
    def isModerated(self):
        """ """
        return self.moderated

    security.declareProtected(View, 'showWizards')
    def showWizards(self):
        """ """
        return self.show_wizards

    security.declareProtected(View, 'forumEmail')
    def forumEmail(self):
        """ """
        return self.forum_email

    security.declareProtected(View, 'quickReplyForm')
    def quickReplyForm(self):
        """ """
        try:
            if self.quickreply_form != 'default':
                return self.quickreply_form
        except:
            pass

        return self.portal_properties.cmfboard_properties.getProperty(
            'quickreply_form', '')

    security.declareProtected(View, 'forumPageSize')
    def forumPageSize(self):
        """ """
        return self.portal_properties.cmfboard_properties.getProperty(
            'forum_page_size', 20)

    security.declareProtected(View, 'topicPageSize')
    def topicPageSize(self):
        """ """
        return self.portal_properties.cmfboard_properties.getProperty(
            'topic_page_size', 20)
    
    def all_meta_types(self, interfaces=None):
        """ """
        return []

    # rename copy support
    _copy = 0
    def _canCopy(self, op=0):
        if op == 1:
            return 1
        else:
            return 0

    def _notifyOfCopyTo(self, *args, **kwargs):
        self._copy = 1
        Referenceable._notifyOfCopyTo(self, *args, **kwargs)

    def manage_afterAdd(self, item, container):
        BaseBTreeFolder.manage_afterAdd(self, item, container)
        self._copy = 0

    security.declareProtected(View, 'manage_simpleSendMessage')
    def manage_simpleSendMessage(self, member_id, title, text):
        """simple_send method from a MailHost Object"""
        mtool = self.portal_membership

        if mtool.isAnonymousUser():
            raise Exception, 'Only members can send emails.'

        member = mtool.getMemberById(member_id)
        if not member:
            raise Exception, "Can't find member."
        mto = member.getProperty('email', '')
        if not mto:
            raise Exception, "Member have not email address."

        sender = mtool.getAuthenticatedMember()
        mfrom = sender.getProperty('email', '')
        if not mfrom:
            raise Exception, "You have not email address."

        sfrom = sender.getProperty('fullname', '')
        if not sfrom:
            sfrom = str(sender)

        self.MailHost.simple_send(mto, '%s <%s>'%(sfrom, mfrom), title, text)

    security.declarePrivate('initializeArchetype')
    def initializeArchetype(self, **kwargs):
        BaseBTreeFolder.initializeArchetype(self, **kwargs)
        
	try:
    	    addForumCatalog(self)
	except:
	    pass

    security.declareProtected(View, 'getForum')
    def getForum(self):
        """ """
        return self


    security.declarePrivate('_addTopic')
    def _addTopic(self, title, text, sticky_flag=0, topic_type=1,
                  portal_type='ForumTopic', **kwargs):
        # fixme need check portal_type
        
        kwargs.update({'title': title, 'text': text, 'topic_type': topic_type})
        if _checkPermission(ManageTopic, self):
            kwargs.update({'sticky_flag': sticky_flag})

        if not title:
            raise Exception, 'Null title'

        id = str(random.random())[2:]
        kwargs['id'] = id

        tp = self.portal_types.getTypeInfo(portal_type)
        if tp is None:
            raise 'ValueError', 'No such content type: ForumTopic'

        m = getattr(self.manage_addProduct['CMFBoard'], tp.factory)

        id = apply(m, (), kwargs)
        ob = tp._finishConstruction(self._getOb(id))

        member = self.portal_membership.getAuthenticatedMember()
        if member.getProperty('fc_autosubscribe', 0):
            ob.Subscribe()

        return ob

    security.declareProtected(AddTopic, 'manage_addTopic')
    def manage_addTopic(self, title, text, sticky_flag=0,
                        portal_type='ForumTopic', **kwargs):
        """ """
        msg = self._addTopic(title, text, sticky_flag,
                             portal_type=portal_type, **kwargs)
        self.sendNotification(msg, None)
        return msg

    security.declareProtected(View, 'manage_addSpecialTopic')
    def manage_addSpecialTopic(self, title, text, tp='admin', **kwargs):
        """ """
        kwargs['sticky_flag'] = 1
        if tp == 'admin':
            return self._addTopic(title, text, topic_type=2, **kwargs)
        elif (tp == 'announce') and _checkPermission(ReviewPortalContent, self):
            return self._addTopic(title, text, topic_type=4, **kwargs)
        else:
            return self

    security.declareProtected(ManageForum, 'notifyStateChanged')
    def notifyStateChanged(self):
        """ """
        catalog = self.forum_catalog
        for brain in catalog():
            ob = brain.getObject()
            if ob is None:
                continue
            s = getattr(ob, '_p_changed', 0)
            catalog.reindexObject(ob, idxs=['allowedRolesAndUsers'])
            if s is None: ob._p_deactivate()
            
        self.reindexObjectSecurity()

    security.declareProtected(View, 'UID')
    def UID(self):
        """ """
        return BaseBTreeFolder.UID(self)

    # remove in future versions
    security.declareProtected(View, 'hasPoll')
    def hasPoll(self):
        """ """
        return has_poll

    security.declarePrivate('upgradeForum')
    def upgradeForum(self):
        """ """
        if (not hasattr(self, '_forum_ver') or self._forum_ver == '1.0'):
            for ob in self.objectValues():
                if ob.getId() != 'forum_catalog':
                    ob.upgradeTopic()
            self._forum_ver = '1.2'

        if self.portal_type != 'ForumNB':
            self.portal_type = 'ForumNB'
            self.portal_catalog.reindexObject(self, ('Type',))

registerType(ForumNB)

Forum = ForumNB
