# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.kz>
__version__ = '$Revision: 1.9 $'[11:-2]


import os, sys, traceback
from Acquisition import aq_base, aq_parent, aq_inner
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager, \
     newSecurityManager, noSecurityManager
from AccessControl.Permission import Permission
from zLOG import LOG, ERROR

import random, time, sys
from Products.CMFCore.utils import UniqueObject, \
     SimpleItemWithProperties, getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import View, ManagePortal
from Products.Archetypes.config import TOOL_NAME
from Products.ZCatalog.ZCatalog import ZCatalog

try:
    import email
    from email import message_from_string
    from email.Utils import parseaddr
    from email.Header import decode_header, make_header
except:
    pass


from ForumPermissions import *

errors = {
    0: "Unknown",
    1: 'Member not found',
    2: 'Forum not found',
    3: 'Topic not found',
    4: "You don't have permission add new topic",
    5: "You not subscribed to forum or topic",
    6: "You don't have permission add new reply",
    7: 'Message not found',
    }

class MailLoader( UniqueObject, SimpleItemWithProperties ):
    """ """
    id = 'portal_mailloader'
    meta_type = 'CMFBoard Mail Loader'

    security = ClassSecurityInfo()

    security.declarePrivate('reply')
    def replyError(self, mto, error):
        self.MailHost.simple_send(mto, self. email_from_address,
                                  'Error', errors.get(error, 'Unknown'))

    security.declareProtected(ManagePortal, 'load')
    def load(self, email):
        """ """
        file = open('/tmp/test_mailloader.log', 'ab')
        #file = sys.stdout
        print >> file, '\n\n', '------------------------------------------------'

        msg = message_from_string(email)

        mtool = getToolByName(self, 'portal_membership')

        mfrom = parseaddr(msg['From'])[1]

        # create new context
        members = mtool.searchMembers('email', mfrom)
        if members:
            member = mtool.getMemberById(members[0]['username'])
        else:
            # member not found
            print >> file, 'member not found', mfrom
            self.replyError(mfrom, error=1)
            return 0

        user = member.getUser()

        to_hdr = parseaddr(msg['To'])[1]

        print >> file, '------- begin loading ----------------'

        try:
            if msg.has_key('In-Reply-To'):
                self.loadReplyTo(user, msg, mfrom, to_hdr, file)
            else:
                self.loadNewTopic(user, msg, mfrom, to_hdr, file)
        except Exception, e:
            print >> file, e
            traceback.print_exc()

        noSecurityManager()

    security.declarePrivate('loadReplyTo')
    def loadReplyTo(self, user, msg, mfrom, mto, file):
        print >> file, 'loadReplyTo'

        # create security context
        portal = aq_parent(aq_inner(self))
        user = user.__of__(portal.acl_users)
        newSecurityManager(self.REQUEST, user)

        # find forum
        reply_to = msg['In-Reply-To']
        try:
            forum_uid, topic_uid, msg_uid = reply_to[1:-1].split('@')[0].split('.')
        except:
            self.loadNewTopic(msg, mfrom, mto, file)

        tool = getToolByName(self, TOOL_NAME)

        forum_uid = forum_uid.replace('_', '.')
        forum = tool.lookupObject(forum_uid)

        print >> file, forum_uid

        if forum is None:
            print >> file, 'Forum not found'
            self.replyError(mfrom, error=2)
            return 'Forum not found'
        if forum.forumEmail() != mto:
            self.replyError(mfrom, error=2)
            print >> file, 'Forum email not found'
            return 'Forum email not found'

        print >> file, forum, topic_uid
        
        topic = tool.lookupObject(topic_uid)
        if topic is None:
            print >> file, 'Topic not found'
            self.replyError(mfrom, error=3)
            return 'Topic not found'

        # check permission "Add forum topic reply"
        if not _checkPermission(AddTopicReply, topic):
            print >> file, topic
            print >> file, "You don't have permission to add reply", AddTopicReply
            self.replyError(mfrom, 6)
            return "You don't have permission to add reply"

        print >> file, "Add reply OK"

        if not (forum.isSubscribed() or topic.isSubscribed()):
            self.replyError(mfrom, 5)
            return 'Not subscribed'

        print >> file, "Subscription OK"
        
        if msg_uid != topic_uid:
            message = tool.lookupObject(msg_uid)
            if message is None:
                print >> file, 'Message not found'
                self.replyError(mfrom, error=7)
                return 'Message not found'
            replyto = message.UID()
        else:
            message = None
            replyto = ''

        title = make_header(
            decode_header(msg.get('Subject').strip())).__unicode__().encode('UTF-8')
        if not title:
            title="[No subject]"

        text, text_format = self.getParsedBody(msg)

        # create reply
        ob = topic.manage_addReply(title, text, replyto, text_format=text_format)
        print >> file, ob, ob.getId()
        print >> file, '----------------------- successful loading ---------------'

    security.declarePrivate('loadNewTopic')
    def loadNewTopic(self, user, msg, mfrom, mto, file):
        print >> file, 'test'

        portal = aq_parent(aq_inner(self))
        user = user.__of__(portal.acl_users)
        newSecurityManager(self.REQUEST, user)

        forums = self.portal_catalog(Type='ForumNB', forum_email=mto)
        if forums:
            forum = forums[0].getObject()
            print >> file, 'Forum found:', forum

            # check permission "Add forum topic"
            if not _checkPermission(AddTopic, forum):
                self.replyError(mfrom, 4)
                return "You don't have permission to add new topic"

            print >> file, "Add forum topic. OK"

            if not forum.isSubscribed():
                self.replyError(mfrom, 5)
                return 'Not subscribed'

            print >> file, "subscription. OK"
                
            title = make_header(
                decode_header(msg.get('Subject').strip())).__unicode__().encode('UTF-8')
            if not title:
                title="[No subject]"

            text, text_format = self.getParsedBody(msg)

            ob = forum.manage_addTopic(title, text, text_format=text_format)
            print >> file, ob, ob.getId()
            print >> file, '----------------------- successful loading ---------------'
        else:
            self.replyError(mfrom, 0)

    security.declarePrivate('get_loads')
    def get_loads(self, message, number=None):
	lst = []
	msgs = message.get_payload()
	type = message.get_type()
	if type == 'application/octet-stream' :
	    lst.append(msgs)
	    return lst

	for msg in msgs :
	    if msg.is_multipart() :
		loads = self.get_loads(msg)
		lst = lst + loads
	    else :
		lst.append(msg)
	if number != None :
	    return lst[number]
	return lst

    security.declarePrivate('getParsedBody')
    def getParsedBody(self, msg):
        """ return parsed body """
        def conv(st, cs):
            charsets = ['koi8-r', 'cp1251', 'iso-8859-1']
            if cs is not None:
                charsets = [cs] + charsets

            for cs in charsets:
                try:
                    return unicode(st, cs, 'replace').encode('utf-8')
                except:
                    pass
            return st
        
        if msg.is_multipart():
            for msg in self.get_loads(msg) :
                type = msg.get_type()
                if type == None :
		    type = 'text/plain'
		if type == 'text/plain':
                    return conv(msg.get_payload(decode=1), msg.get_content_charset()), 'text/plain'
		if type == 'text/html':
		    return conv(msg.get_payload(decode=1), msg.get_content_charset()), 'html'

            return '', ''
        else:
            st = msg.get_payload(decode=1)
            if st is None: return '', 'text/plain'
            st = conv(st, msg.get_content_charset())
            
	    st = email.quopriMIME.decode(st)
            
	    if msg.get_type() == 'text/html':
                return st, 'html'
	    else:
                return st, 'text/plain'



InitializeClass(MailLoader)
