# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
__version__ = '$Revision: 1.8 $'[11:-2]

from Globals import InitializeClass
from Products.Archetypes.public import *
from Products.CMFCore.CMFCorePermissions import View
from Products.MPoll.MPoll import MPoll

from config import *
from ForumTopic import ForumTopic
from ForumPermissions import AddTopicReply, ModifyTopic

class ForumPoll(ForumTopic, MPoll):
    """ forum poll topic based on MPoll product """
    meta_type = 'ForumPoll'
    archetype_name = 'Forum Poll'

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []
    content_icon = 'fcPoll_icon.gif'
    content_icon_norm = 'fcPoll_icon.gif'

    schema = ForumTopic.schema + Schema((
        LinesField('answers', required=1, searchable=1,
                   write_permission=ModifyTopic,
                   widget=LinesWidget(description="A list of alternatives. One alternative per row.",
                                      label_msgid="label_answers",
                                      description_msgid="help_answers",
                                      i18n_domain="mpoll"),
                   ),
        BooleanField('open', accessor='isOpen', mutator='setOpen', default=1,
                     write_permission=ModifyTopic,
                     widget=BooleanWidget(description="Check this option to make the poll open, accepting votes from users.",
                                          label_msgid="label_open",
                                          description_msgid="help_open",
                                          i18n_domain="mpoll"),
                     ),
        ))
    
    if plone_ver == 1:
        actions = (
            {'id': 'view',
             'name': 'View',
             'action': 'forum_topic',
             'permissions': (View,)
             },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'portal_form/forum_poll_edit',
              'permissions': (ModifyTopic,)
              },
            {'id': 'preview',
             'name': 'Preview',
             'action': 'forum_message_preview',
             'permissions': (View,)
             },
            )
    else:
        actions = (
            {'id': 'view',
             'name': 'View',
             'action': 'string:$object_url/forum_topic',
             'permissions': (View,)
             },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/portal_form/forum_poll_edit',
              'permissions': (ModifyTopic,)
              },
            {'id': 'preview',
             'name': 'Preview',
             'action': 'string:$object_url/forum_message_preview',
             'permissions': (View,)
             },
            )


    security = ClassSecurityInfo()

    # poll
    topic_type = 3

    def __init__(self, oid, **kwargs):
        ForumTopic.__init__(self, oid, **kwargs)
        MPoll.__init__(self, oid, **kwargs)

    security.declareProtected(View, 'getQuestion')
    def getQuestion(self):
        """ """
        return self.title

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self):
        """ """
        msg = ForumTopic.CookedBody(self)
        return self.forum_poll_view(message=msg)

    security.declareProtected(AddTopicReply, 'vote')
    vote = MPoll.vote

registerType(ForumPoll)
InitializeClass(ForumPoll)
