# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz>
__version__ = '$Revision: 1.34 $'[11:-2]

import random, traceback, copy
from Globals import InitializeClass
from DateTime import DateTime
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_parent, aq_base
from Products.Archetypes.public import *
from Products.Archetypes.config import TOOL_NAME
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, ManageProperties, ManagePortal, RequestReview
from Products.CMFCore.utils import getToolByName

from config import *
from ForumMessage import ForumMessage
from ForumPermissions import *
from Subscribeable import Subscribeable
from ForumFileField import ForumFileField

schema = BaseSchema + Schema((
    StringField('title', required=1,
                write_permission=ModifyTopic,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),
    StringField('content', default='',
                write_permission=ModifyTopic,
                widget=TextAreaWidget(label='Content')),
    ImageField('image',
               validators=('checkImageSize',),
               write_permission=ModifyTopic,
               allowable_content_types=('image/*',),
               sizes={'thumbnail': (128, 128),},
               widget=ImageWidget(label='Image')),
    ForumFileField('file',
                   validators=('checkFileSize',),
                   write_permission=ModifyTopic,
                   widget=FileWidget(label='File')),
    StringField('message_icon',
                write_permission=ModifyTopic,
                widget=StringWidget(label='Message icon')),
    BooleanField('sticky_flag', default=0,
                 mutator='_change_sticky_flag',
                 write_permission=ManageTopic,
                 widget=BooleanWidget(label='Sticky')),
    StringField('text_format', default='text/plain',
                required=0,
                write_permission=ModifyTopic,),
    StringField('text', default='text/plain',
                accessor='EditableBody',
                mutator='setText',
                write_permission=ModifyTopic,
                widget=VisualWidget(label='Body text')),
    IntegerField('signature', default=0,
                 write_permission=ModifyReply,
                 widget=SelectionWidget()),
    ))


class ForumTopic(BaseBTreeFolder, ForumMessage, Subscribeable):
    """ """
    meta_type = 'ForumTopic'
    archetype_name = 'Forum Topic'

    schema = schema

    global_allow = 0
    filter_content_types = 1
    allowed_content_types = []

    if plone_ver == 1:
        actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'forum_topic',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'portal_form/forum_message_edit',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'portal_form/forum_reply_form',
              'permissions': (View,)
              },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'forum_message_preview',
              'permissions': (View,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'portal_form/forum_message_delete_form',
              'permissions': (ModifyTopic,)
              },
            )
    else:
        actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/forum_topic',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/portal_form/forum_message_edit',
              'permissions': (ModifyTopic,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/portal_form/forum_reply_form',
              'permissions': (View,)
              },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (View,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/portal_form/forum_message_delete_form',
              'permissions': (ModifyTopic,)
              },
            )

    content_icon = 'fcTopic_icon.gif'
    content_icon_norm = 'fcTopic_icon.gif'

    security = ClassSecurityInfo()
    security.setPermissionDefault(RequestReview, [replyOwner])

    _created = 0
    last_reply_id = ''
    last_reply_user = ''
    hits = 0
    replies = 0

    _canCopy = ForumMessage._canCopy
    Creator = ForumMessage.Creator

    topic_type = 1

    last_reply_user = None
    description = None
    signature = 0

    security.declareProtected(ModifyTopic, 'setText')
    security.declareProtected(ModifyTopic, 'processForm')

    def __init__(self, oid, **kwargs):
        BaseBTreeFolder.__init__(self, oid, **kwargs)
        ForumMessage.__init__(self, oid, **kwargs)

    def _change_sticky_flag(self, value):
        try:
            self.sticky_flag = int(value)
        except:
            self.sticky_flag = 0
        self.modification_date = DateTime()
        self.reindexObject(['sticky_flag', 'modified'])

    def _change_locked_flag(self, value):
        self.locked_flag = value
        self.reindexObject(['locked_flag'])

    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('topic_type'): # and kwargs['topic_type'] in topicTypeList:
            self.topic_type = kwargs['topic_type']

        BaseBTreeFolder.initializeArchetype(self, **kwargs)
        ForumMessage.initializeArchetype(self, **kwargs)

        if self.last_reply_user == None:
            self.last_reply_user = self.Creator()
            self.description = self.Content(add_breaks=0, preview=1)[:150]

    def all_meta_types(self, interfaces=None):
        """ """
        return []

    _set_owner = 0
    def manage_afterAdd(self, item, container):
        BaseBTreeFolder.manage_afterAdd(self, item, container)
        ForumMessage.manage_afterAdd(self, item, container)

        self.manage_permission(ModifyPortalContent, (), 0)

        # update member posts count
        if not (hasattr(item, '_copy') and item._copy):
            mtool = self.portal_membership
            user = mtool.getAuthenticatedMember()

            if not mtool.isAnonymousUser() and user:
                self.manage_permission(RequestReview, (topicOwner,), 0)
                self.manage_setLocalRoles(user.getId(), [topicOwner,])

    def manage_beforeDelete(self, item, container):
        BaseBTreeFolder.manage_beforeDelete(self, item, container)
        ForumMessage.manage_beforeDelete(self, item, container)

    security.declareProtected(AddTopicReply, 'manage_addReply')
    def manage_addReply(self, title='', text='', reply_to='', **kwargs):
        """ """
        if not title:
            title = self.title

        id = str(random.random())[2:]
        
        tool = getToolByName(self, TOOL_NAME)
        reply_to = tool.lookupObject(reply_to)
        if reply_to is not None:
            parent = reply_to
        else:
            parent = self.LastReply()

        kwargs.update({'id':id, 'parent':parent, 'title':title, 'text': text})

        tp = self.portal_types.getTypeInfo('ForumMessage')
        if tp is None:
            raise 'ValueError', 'No such content type: ForumMessage'

        m = getattr(self.manage_addProduct['CMFBoard'], tp.factory)
        id = apply(m, (), kwargs)
        msg = tp._finishConstruction(self._getOb(id))

        # update info about last reply
        self.replies += 1
        self.modification_date = msg.created()

        self.sendNotification(msg, parent)
        self.reindexObject(['replies', 'modified'])
        return msg

    def _delObject(self, id, dp=1):
        self.replies -= 1
        BaseBTreeFolder._delObject(self, id, dp)

    security.declareProtected('Delete objects', 'manage_delObjects')
    def manage_delObjects(self, ids=[], REQUEST=None):
        """ """
        return self.manage_deleteReply(ids, REQUEST)

    security.declareProtected(DeleteTopic, 'manage_deleteMessage')
    def manage_deleteMessage(self, REQUEST=None):
        """ """
        try:
            aq_parent(self).manage_delObjects([self.getId()])
            msg = '/?portal_status_message=Topic+deleted'
            if REQUEST is not None:
                return REQUEST.RESPONSE.redirect('%s%s'%(
                    aq_parent(self).absolute_url(), msg))
        except Exception, e:
            msg = '/?portal_status_message=%s'%e
            if REQUEST is not None:
                return REQUEST.RESPONSE.redirect('%s%s'%(self.absolute_url(), msg))
        
    security.declarePrivate('manage_deleteReply')
    def manage_deleteReply(self, ids, REQUEST=None):
        """ """
        if type(ids) == type(''):
            ids = [ids]

        BaseBTreeFolder.manage_delObjects(self, ids)

        self.setLastReply()

        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect('%s/?portal_status_message=Reply+deleted'%(
                self.absolute_url()))

    security.declarePrivate('setLastReply')
    def setLastReply(self, ob=None):
        """ """
        if ob is not None:
            new_id = ob.getId()
            if new_id != self.last_reply_id:
                self.last_reply_id = new_id
                self.last_reply_user = ob.Creator()
                self.reindexObject(['replies', 'last_reply_user'])
        else:
            try:
                catalog = getToolByName(self, 'portal_catalog')
                brains = catalog(
                    review_state=('published', 'visible'),
                    Type='Forum Message', sort_on='modified',
                    sort_order='reverse', path='/'.join(self.getPhysicalPath()))
                ob = brains[0].getObject()
                new_id = ob.getId()
                if new_id != self.last_reply_id:
                    self.last_reply_id = new_id
                    self.last_reply_user = ob.Creator()
                    self.reindexObject(['replies', 'last_reply_user'])
            except:
                ob = self
                self.last_reply_id = ''
                self.last_reply_user = self.Creator()
                self.reindexObject(['replies', 'last_reply_user'])

        self.description = ob.Content(add_breaks=0, preview=1)[:150]

    security.declarePrivate('updateLastReply')
    def updateLastReply(self, ob):
        """ """
        new_id = ob.getId()
        if new_id == self.last_reply_id:
            self.description = ob.Content(add_breaks=0, preview=1)[:150]
            
    security.declareProtected(View, 'LastReply')
    def LastReply(self):
        """ return last reply """
        id = self.last_reply_id
        if id:
            try:
                return getattr(self, id)
            except:
                pass
        return self

    security.declareProtected(View, 'Hits')
    def Hits(self):
        """ return hits and inc """
        hits = self.hits + 1
        self.hits = hits
        #self.reindexObject(['hits'])
        return hits

    security.declareProtected(View, 'getTopic')
    def getTopic(self):
        """ """
        return self

    Topic = getTopic

    security.declarePublic('getTopicType')
    def getTopicType(self):
        """ """
        return self.topic_type

    security.declareProtected(View, 'replyCount')
    def replyCount(self, *args, **kwargs):
        """ for compatibility with default discussion tool """
        return self.replies

    ##########################################
    # forum_catalog support
    ##########################################
    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        BaseContent.indexObject(self)
        self.forum_catalog.indexObject(self)

    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self):
        BaseContent.unindexObject(self)
        self.forum_catalog.unindexObject(self)

    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        BaseContent.reindexObject(self, idxs)
        self.forum_catalog.reindexObject(self, idxs=idxs)

    security.declarePrivate('upgradeTopic')
    def upgradeTopic(self):
        """ """
        if hasattr(aq_base(self), '_children'):
            for id in self._children:
                child = getattr(self, id, None)
                if child is not None:
                    child.addReference(self, 'parent')
            del self._children

    security.declareProtected(View, 'notifyStateChanged')
    def notifyStateChanged(self, sci):
        """ """
        try:
            object = sci.object
            status = sci.status['review_state'], 
            if object.Type() == 'Forum Message':
                if status[0] in ('visible','published'):
                    self.setLastReply(object)
                elif object.getId() == self.last_reply_id:
                    object.reindexObject(('review_state','modified'))
                    self.setLastReply()
        except:
            traceback.print_exc()

registerType(ForumTopic)
InitializeClass(ForumTopic)
