# -*- Mode: Python; py-indent-offset: 4 -*-
# * Copyright (C) 2003 Nikolay Kim <fafhrd@legco.biz> 

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.Permission import Permission

from ForumPermissions import *

managedablePermissions = [ View, AddTopic, AddTopicReply ]
managedablePermissions.sort()
managedablePermissions = tuple(managedablePermissions)

class PermissionsManagement:

    security = ClassSecurityInfo()

    security.declareProtected(ManageForum, 'getForumGroups')
    def getForumGroups(self):
        """ """
        try:
            groups = self.portal_groups
        except:
            return []

        res = []
        for grp in groups.listGroups():
            res.append((grp.getUserName(), grp.getUserNameWithoutGroupPrefix()))
            
        return res

    # permissions management API
    security.declareProtected(ManageForum, 'getForumRoles')
    def getForumLocalRoles(self):
        """ """
        acl = self.acl_users

        if acl.meta_type == 'Group User Folder':
            return acl.getLocalRolesForDisplay(self)
        else:
            results = []
            for rec in self.get_local_roles():
                results.append(rec + ('user', rec[0]))
            return results

    security.declareProtected(ManageForum, 'delForumRoles')
    def delForumLocalRoles(self, member_ids):
        """ """
        if type(member_ids) == type(''):
            member_ids = (member_ids,)
            
        self.manage_delLocalRoles( userids = member_ids )

    security.declareProtected(ManageForum, 'setForumRoles')
    def setForumLocalRoles(self, member_ids, role=None):
        """ """
        if role is None:
            return

        role = role_map[int(role)]

        if type(member_ids) == type(''):
            member_ids = (member_ids,)
        
        for member_id in member_ids:
            self.manage_setLocalRoles( member_id, [role,] )

        # update catalog
        self.notifyStateChanged()


    # roles permissions management API
    security.declareProtected(ManageForum, 'getForumRoles')
    def getForumRoles(self):
        """ """
        roles = self.portal_membership.getCandidateLocalRoles(self)

        for r in ('Owner', 'Reviewer', 'Manager'):
            try:
                del roles[roles.index(r)]
            except:
                pass

        if 'Anonymous' not in roles:
            roles = ['Anonymous'] + roles

        return roles


InitializeClass(PermissionsManagement)
