# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 

import re, traceback, binascii
from types import StringType
from Acquisition import aq_parent, aq_inner
from ExtensionClass import Base
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.PermissionRole import rolesForPermissionOn
from AccessControl.SecurityManagement import getSecurityManager, \
     newSecurityManager, noSecurityManager

from BTrees.OIBTree import OIBTree
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFCore.CMFCorePermissions import View

from ForumPermissions import ManageForum

isEmail = re.compile('^[0-9a-zA-Z_&.%+-]+@([0-9a-zA-Z]([0-9a-zA-Z-]*[0-9a-zA-Z])?\.)+[a-zA-Z]{2,6}$')


class Subscribeable(Base):
    """ """

    security = ClassSecurityInfo()

    security.declareProtected(ManageForum, 'getSubscriber')
    def getSubscriber(self, username):
        """ """
        try:
            return self._subscription[username]
        except:
            return ''

    security.declareProtected(ManageForum, 'getSubscribersList')
    def getSubscribersList(self):
        """ """
        return list(self._subscription.keys())

    security.declareProtected(ManageForum, 'getSubscribersList')
    def removeSubscribers(self, members):
        """ """
        if type(members) == type(''):
            members = [members,]

        subs = self._subscription
        for m in members:
            try:
                del subs[m]
            except:
                pass

    security.declareProtected(View, 'isSubscribed')
    def isSubscribed(self):
        """ """
        try:
            subs = self._subscription
        except AttributeError, e:
            subs = OIBTree()
            self._subscription = subs

        mtool = self.portal_membership

        if mtool.isAnonymousUser():
            return 0

        user = mtool.getAuthenticatedMember()
        if not user.getProperty('email', ''):
            self.Unsubscribe()
            return 0
            
        user_name = user.getUserName()
        if subs.has_key(user_name) and subs[user_name]:
            return 1
        else:
            return 0

    security.declareProtected(View, 'Subscribe')
    def Subscribe(self):
        """ """
        try:
            subs = self._subscription
        except AttributeError, e:
            subs = OIBTree()
            self._subscription = subs

        mtool = self.portal_membership

        if mtool.isAnonymousUser():
            return 0

        user = mtool.getAuthenticatedMember()
        user_name = user.getUserName()
        if not user.getProperty('email', ''):
            raise Exception, 'email not found'
        
        subs[user_name] = 1

    security.declareProtected(View, 'Unsubscribe')
    def Unsubscribe(self):
        """ """
        subs = self._subscription
        mtool = self.portal_membership

        if mtool.isAnonymousUser():
            return 0

        user = mtool.getAuthenticatedMember()
        user_name = user.getUserName()
        try:
            del subs[user_name]
        except:
            pass


    security.declarePrivate('genMessageId')
    def genMessageId(self, msg):
        """ """
        forum_uid = msg.getForum().UID()
        forum_uid = forum_uid.replace('.', '_')
        
        topic_uid = msg.getTopic().UID()
        
        try:
            msg_id = msg.getMessage().UID()
        except:
            msg_id = forum_uid

        try:
            host = self.email_from_address.split('@')[1]
        except:
            host = 'localhost'
                
        s = '<%s.%s.%s@%s>' % (forum_uid, topic_uid, msg_id, host)
        return s


    def _getSubscribers(self, message, topic, portal, mtool):
        try:
            users = aq_parent(aq_inner(self))._getSubscribers(message, topic, portal, mtool)
        except:
            users = {}

        try:
            subs = self._subscription
        except AttributeError, e:
            subs = OIBTree()
            self._subscription = subs

        for user_id, state in subs.items():
            if not users.has_key(user_id):
                member = mtool.getMemberById(user_id)

                if not member:
                    continue
                    
                email = member.getProperty('email')

                if isEmail.search(email.lower()) != None:
                    # check if user can view this message
                    user = member.getUser()
                    user = user.__of__(portal.acl_users)
                    newSecurityManager(None, user)
                    if _checkPermission(View, topic):
                        users[user_id] = email
                    noSecurityManager()
                    
        return users

    security.declarePrivate('sendNotification')
    def sendNotification(self, message, parent):
        """ send reply to all subsribed users """

        portal = getToolByName(self, 'portal_url').getPortalObject()
        mtool = portal.portal_membership

        try:
            mspool = self.portal_mailspool
        except:
            return

        headers = ''

        email_from_address=self.email_from_address

        user = mtool.getMemberById(message.Creator())
        if user:
            fullname = user.getProperty('fullname', str(user))
            if not fullname:
                fullname = str(user)
            
            email = user.getProperty('email', email_from_address)
            if not email:
                email = email_from_address

            headers += 'From: %s <%s>\n' % (fullname, email)
        else:
            headers += 'From: Anonymous <%s>\n' % email_from_address

        headers += 'Message-ID: %s\n'%self.genMessageId(message)
        if parent:
            headers += 'In-Reply-To: %s\n'%self.genMessageId(parent)

        forum = self.getForum()
        forumEmail = self.forumEmail()
        if forumEmail:
            headers += 'Reply-To: %s\n'%forumEmail
            headers += 'List-Id: %s\n\t%s\n'%(forum.Title(), forumEmail.replace('@', '.'))
            headers += 'List-Post: <mailto:%s>\n'%forumEmail
            headers += 'List-Unsubscribe: <%s>\n'%forum.absolute_url()
            headers += 'List-Subscribe: <%s>\n'%forum.absolute_url()
            headers += 'List-Archive: <%s>\n'%forum.absolute_url()

        users = self._getSubscribers(message, aq_inner(message.getTopic()), portal, mtool)

        if not users: return

        charset = self.portal_properties.site_properties.default_charset

        headers += ('BCC: %s\n'%','.join(users.values()))
        if message.text_format in ('plain', 'text/plain'):
            headers += 'Content-Type: text/plain; charset=%s\n\n\n'%charset
            body = headers + message.EditableBody()
        else:
            headers += 'Content-Type: text/html; charset=%s\n\n\n'%charset

            user_id = users.keys()[0]
            user = portal.acl_users.getUser(user_id)
            user = user.__of__(portal.acl_users)
            newSecurityManager(None, user)
            body = headers + message.forum_email_template()
            noSecurityManager()

        try:
            title = '=?%s?B?%s?=' % (
                charset, binascii.b2a_base64(message.Title() or '[No subject]').strip())
            mspool.addMessage((body, '', None, title, 'quoted-printable'))
        except Exception, e:
            traceback.print_exc()


InitializeClass(Subscribeable)
